//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/SampleViewFactory.cpp
//! @brief     Implements class SampleViewFactory
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/SampleDesigner/SampleViewFactory.h"
#include "GUI/Models/InterferenceItems.h"
#include "GUI/Models/LayerItem.h"
#include "GUI/Models/MesoCrystalItem.h"
#include "GUI/Models/MultiLayerItem.h"
#include "GUI/Models/ParticleCompositionItem.h"
#include "GUI/Models/ParticleCoreShellItem.h"
#include "GUI/Models/ParticleItem.h"
#include "GUI/Models/ParticleLayoutItem.h"
#include "GUI/Models/TransformationItem.h"
#include "GUI/Views/SampleDesigner/InterferenceViews.h"
#include "GUI/Views/SampleDesigner/LayerView.h"
#include "GUI/Views/SampleDesigner/MesoCrystalView.h"
#include "GUI/Views/SampleDesigner/MultiLayerView.h"
#include "GUI/Views/SampleDesigner/ParticleCompositionView.h"
#include "GUI/Views/SampleDesigner/ParticleCoreShellView.h"
#include "GUI/Views/SampleDesigner/ParticleLayoutView.h"
#include "GUI/Views/SampleDesigner/ParticleView.h"
#include "GUI/Views/SampleDesigner/TransformationView.h"

namespace {

QStringList valid_item_types = {MultiLayerItem::M_TYPE,
    LayerItem::M_TYPE,
    ParticleLayoutItem::M_TYPE,
    ParticleItem::M_TYPE,
    TransformationItem::M_TYPE,
    ParticleCoreShellItem::M_TYPE,
    ParticleCompositionItem::M_TYPE,
    MesoCrystalItem::M_TYPE,
    Interference1DLatticeItem::M_TYPE,
    Interference2DLatticeItem::M_TYPE,
    Interference2DParaCrystalItem::M_TYPE,
    InterferenceFinite2DLatticeItem::M_TYPE,
    InterferenceHardDiskItem::M_TYPE,
    InterferenceRadialParaCrystalItem::M_TYPE};

} // namespace

bool SampleViewFactory::isValidType(const QString& model_type)
{
    if (model_type.startsWith("FormFactor")) {
        return true;
    }
    return valid_item_types.contains(model_type);
}

IView* SampleViewFactory::createSampleView(const QString& model_type)
{
    if (model_type == MultiLayerItem::M_TYPE) {
        return new MultiLayerView();
    } else if (model_type == LayerItem::M_TYPE) {
        return new LayerView();
    } else if (model_type == ParticleLayoutItem::M_TYPE) {
        return new ParticleLayoutView();
    } else if (model_type == ParticleItem::M_TYPE) {
        return new ParticleView();
    } else if (model_type == TransformationItem::M_TYPE) {
        return new TransformationView();
    } else if (model_type == ParticleCoreShellItem::M_TYPE) {
        return new ParticleCoreShellView();
    } else if (model_type == ParticleCompositionItem::M_TYPE) {
        return new ParticleCompositionView();
    } else if (model_type == MesoCrystalItem::M_TYPE) {
        return new MesoCrystalView();
    } else if (model_type == Interference1DLatticeItem::M_TYPE) {
        return new Interference1DLatticeView();
    } else if (model_type == Interference2DLatticeItem::M_TYPE) {
        return new Interference2DLatticeView();
    } else if (model_type == Interference2DParaCrystalItem::M_TYPE) {
        return new Interference2DParaCrystalView();
    } else if (model_type == InterferenceFinite2DLatticeItem::M_TYPE) {
        return new InterferenceFinite2DLatticeView();
    } else if (model_type == InterferenceHardDiskItem::M_TYPE) {
        return new InterferenceHardDiskView();
    } else if (model_type == InterferenceRadialParaCrystalItem::M_TYPE) {
        return new InterferenceRadialParaCrystalView();
    }
    return 0;
}
